
#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include <tap.h>
#include "exTap.h"
#include "tf5800.h"

//-----------------------------------------------------------------------------
extern void oprintf(const void *fmt, ...);
extern int playing_chNum;
extern int sub_chNum;

typedef struct{
    int recType;            //0=none, 1=rec, 2=timeshift
    int svcType;
    int slot;
    int channel_number;
} type_srpids;
extern type_srpids srpid[32];

//-----------------------------------------------------------------------------
int debug = -1;

int on_playback_started();
int on_playback_stopped();
int on_timeshift_playback_started();
int on_timeshift_playback_stopped();
int on_recording_playback_started();
int on_recording_playback_stopped();
int on_channel_change();
int on_record_started(int slot);
int on_record_stopped(int slot);
int on_timeshift_record_started(int slot);
int on_timeshift_record_stopped(int slot);
void two_second_update();

//-----------------------------------------------------------------------------
int on_playback_started()
{
    TYPE_PlayInfo pi;        //overflows!
    byte dummy2[0x80];

    TAP_Hdd_GetPlayInfo(&pi);
    playing_chNum = pi.svcNum;
    sub_chNum = pi.svcNum;
    if (debug & 2) TAP_Print("\nPlayback Started");
    return 0;
}

int on_playback_stopped()
{
    int channel_type, channel_number;

    TAP_Channel_GetCurrent(&channel_type, &channel_number);
    playing_chNum = channel_number;
    sub_chNum = channel_number;
    if (debug & 2) TAP_Print("\nPlayback Stopped");
    return 0;
}


int on_timeshift_playback_started()
{
    TYPE_PlayInfo pi;        //overflows!
    byte dummy2[0x80];

    TAP_Hdd_GetPlayInfo(&pi);
    playing_chNum = pi.svcNum;
    sub_chNum = pi.svcNum;
    if (debug & 2) TAP_Print("\nTimeshift Playback Started");
    return 0;
}

int on_timeshift_playback_stopped()
{
    int channel_type, channel_number;

    TAP_Channel_GetCurrent(&channel_type, &channel_number);
    playing_chNum = channel_number;
    sub_chNum = channel_number;
    if (debug & 2) TAP_Print("\nTimeshift Playback Stopped");
    return 0;
}


int on_recording_playback_started()
{
    TYPE_PlayInfo pi;        //overflows!
    byte dummy2[0x80];

    TAP_Hdd_GetPlayInfo(&pi);
    playing_chNum = pi.svcNum;
    sub_chNum = pi.svcNum;
    if (debug & 2) TAP_Print("\nRecording Playback Started");
    return 0;
}

int on_recording_playback_stopped()
{
    int channel_type, channel_number;

    TAP_Channel_GetCurrent(&channel_type, &channel_number);
    playing_chNum = channel_number;
    sub_chNum = channel_number;
    if (debug & 2) TAP_Print("\nRecording Playback Stopped");
    return 0;
}




//or anything else that causes the timeshift buffer to be lost
int on_channel_change()  // == timeshift killed
{
    int channel_type, channel_number;

    TAP_Channel_GetCurrent(&channel_type, &channel_number);
    playing_chNum = channel_number;
    sub_chNum = channel_number;
    if (debug & 2) TAP_Print("\nChannel Change, ch %d", channel_number);
    return 0;
}





int on_record_started(int slot)
{
    if (debug & 2) TAP_Print("\nRecord Started, slot=%x", slot);
    return 0;
}

int on_record_stopped(int slot)
{
    if (debug & 2) TAP_Print("\nRecord Stopped, slot=%x", slot);

    //disable any user added pids
    on_record_stop(slot);
    return 0;
}


int on_timeshift_record_started(int slot)
{
    if (debug & 2) TAP_Print("\nTimeshift Record Started, slot=%x", slot);
    return 0;
}

int on_timeshift_record_stopped(int slot)
{
    if (debug & 2) TAP_Print("\nTimeshift Record Stopped, slot=%x", slot);
    return 0;
}

//-----------------------------------------------------------------------------
void two_second_update()
{
    TYPE_TapChInfo ci;
    int tvRadio;
    int chNum;
    static int last_tvRadio = -1;
    static int last_chNum = -1;
    static int last_sub_chNum = -1;

    TYPE_PlayInfo pi;        //overflows!
    byte dummy2[0x80];
    static int last_playMode = -1;

    int slot;
    TYPE_RecInfo ri;
    byte dummy3[0x80];
    static int last_recType[2] = {-1, -1};

    int i;
    int emma_pid, active, slot0, slot1;
    int pid;


    //channel changed
    TAP_Channel_GetCurrent(&tvRadio, &chNum);
    if (!((tvRadio == last_tvRadio) && (chNum == last_chNum))) {
        on_channel_change();  // == timeshift killed
        if (debug & 2) {
            if (last_tvRadio < 0) {
                TAP_Print("\nNone");
            }
            else {
                TAP_Channel_GetInfo(last_tvRadio, last_chNum, &ci);
                TAP_Print("\n%s:%d.", (last_tvRadio==SVC_TYPE_Tv)?"tv":"radio", last_chNum, ci.logicalChNum);
            }
            TAP_Print(" %s", ci.chName);
            TAP_Channel_GetInfo(tvRadio, chNum, &ci);
            TAP_Print(" to %s:%d.", (tvRadio==SVC_TYPE_Tv)?"tv":"radio", ci.logicalChNum);
            TAP_Print(" %s", ci.chName);
        }
        last_tvRadio = tvRadio;
        last_chNum = chNum;
    }

    //sub channel changed
    TAP_Channel_GetCurrentSubTV(&chNum);
    if (chNum != last_sub_chNum) {
        last_sub_chNum = chNum;
        sub_chNum = chNum;
    }

    //playmode changed
    TAP_Hdd_GetPlayInfo(&pi);
    if (pi.playMode != last_playMode) {
        if (debug & 2) TAP_Print("\nPlayMode: %s to %s", get_str(playmode_str, last_playMode), get_str(playmode_str, pi.playMode));
        if (last_playMode >= 0)
        switch(last_playMode) {
            case PLAYMODE_None:
                break;
            case PLAYMODE_Playing:
                on_playback_stopped();
                break;
            case PLAYMODE_TempPlaying:
                on_timeshift_playback_stopped();
                break;
            case PLAYMODE_RecPlaying:
                on_recording_playback_stopped();
                break;
        }
        switch(pi.playMode) {
            case PLAYMODE_None:
                break;
            case PLAYMODE_Playing:
                on_playback_started();
                break;
            case PLAYMODE_TempPlaying:
                on_timeshift_playback_started();
                break;
            case PLAYMODE_RecPlaying:
                on_recording_playback_started();
                break;
        }
        last_playMode = pi.playMode;
    }






    //recmode changed
    for (slot=0; slot<2; slot++) {
        TAP_Hdd_GetRecInfo(slot, &ri);
        if (ri.recType != last_recType[slot]) {
        if (debug & 2) TAP_Print("\nRecType[%x]: %s to %s", slot, get_str(rectype_str, last_recType[slot]), get_str(rectype_str, ri.recType));
            if (last_recType[slot] >= 0)
            switch(last_recType[slot]) {
                case RECTYPE_None:
                    break;
                case RECTYPE_Normal:
                    on_record_stopped(slot);
                    break;
                case RECTYPE_Timeshift:
                    on_timeshift_record_stopped(slot);
                    break;
                case RECTYPE_Copy:
                    break;
            }
            switch(ri.recType) {
                case RECTYPE_None:
                    break;
                case RECTYPE_Normal:
                    on_record_started(slot);
                    break;
                case RECTYPE_Timeshift:
                    on_timeshift_record_started(slot);
                    break;
                case RECTYPE_Copy:
                    break;
            }
            last_recType[slot] = ri.recType;
        }
    }





    //check what is recording
    for (i=0; i<32; i++) {
        emma_pid = *(dword *)(EMMA_TTSD_SR_PID0+(i*4));
        active = (emma_pid & 0x00020000) ? 1 : 0;
        slot0 = (emma_pid & 0x00010000) ? 1 : 0;
        slot1 = (emma_pid & 0x00002000) ? 1 : 0;
        pid = emma_pid & 0x1fff;
        if (active) {
            if (srpid[i].recType == RECTYPE_None) {
//                TAP_Print("\noops, need to add");
//                TAP_Print(" *** %8x : slot0 = %d, slot1 = %d, EMMA_TTSD_SR_PID%-2d = %04x", emma_pid, slot0, slot1, i, pid);

                srpid[i].recType = 3;   //f/w added
                srpid[i].svcType = 0;
                srpid[i].slot = slot1;
                srpid[i].channel_number = -1;
            }

        }
        else {
            if (srpid[i].recType != RECTYPE_None) {
//                TAP_Print("\noops, need to remove");
//                TAP_Print(" ***, i=%d, recType=%x, slot=%x, ch=%d", i, srpid[i].recType, srpid[i].slot, srpid[i].channel_number);
//                TAP_Print("\n    %8x : slot0 = %d, slot1 = %d, EMMA_TTSD_SR_PID%-2d = %04x", emma_pid, slot0, slot1, i, pid);

                srpid[i].recType = 0;
                srpid[i].svcType = 0;
                srpid[i].slot = 0;
                srpid[i].channel_number = 0;
            }
        }
    }

}

